/* @file */

/*! \file  mma8452.c:
 *  \brief 
 *  I2C based access library for the MMA8452 accelerometer
 *  using pigpio for I2C access<br>
 *  Copyright U. Raich Nov. 2018<br>
 *  This program is part of a course on embedded systems held
 *  at the University of Cape Coast, Ghana in 2018<br>
 *  It is released under the GNU Public License 
 *  For details please see https://www.gnu.org/licenses/gpl.html
 *  \author Uli Raich
 */
#include <stdio.h>
#include "mma8452.h"

int handle,pi; /* file descriptor for the i2c device */
bool mma8452Debug = true;

unsigned char temp;
char *mm8452DataRangeString[] = {
  "1.25 ms",
  "2.5 ms",
  "5 ms",
  "10 ms",
  "20 ms",
  "80 ms",
  "160 ms",
  "640 ms",
};

char *mm8452ErrorString[] = {
  "success",
  "pigpio init failure",
  "I2C init failure",
  "mma8452 not found",
  "invalid parameter",
  "not standby",
};

char *mma8452ModeString[] = {
  "standby",
  "wake",
  "sleep",
};

char *mma8452RangeString[] = {
  "2g","4g","8g",
};

char *mma8452OrientationString[] = {
  "portrait up",
  "portrait down",
  "landscape up",
  "landscape down",
};

/*! @brief switching debug messages on or off
    @param onOff: to switch debug mode on or off
 */
void mma8452SetDebug(bool onOff) {
  mma8452Debug = onOff;
}

/*! @brief Setup the accelerometer on the I2C bus of the RPi
 * Opens the connection to the pigoio daemon
 * @retval error_code: MMA8452_SUCCESS if successful,
 *                     MMA8452_PIGPIO_INIT_FAILURE otherwise
*/
int mma8452Setup(void)
{
  int retCode;
 /*
    get access to the pigpio daemon 
 */
    if ((retCode = pigpio_start(NULL,NULL)) < 0) {
    printf("mma8452Setup: Cannot initialize pigpio\n");
    return MMA8452_PIGPIO_INIT_FAILURE;
  }
  else
    pi=retCode;

  if (mma8452Debug)
    printf("mma8452Setup: pigpio successfully initialized\n");
  /*
    open the i2c bus
  */
    if ((retCode = i2c_open(pi,I2C_BUS,MMA8452_ADDRESS,I2C_FLAGS)) < 0) {
    printf("mma8452Setup: Cannot open I2C bus %d at address %02x\n",
	   I2C_BUS,MMA8452_ADDRESS);
    if ((pigifError_t)retCode == pigif_bad_send) {
      printf("mma8452Setup: Did you start the pigpiod daemon?\n");
      return MMA8452_I2C_INIT_FAILURE;
    }
  }
  handle = retCode;
  if (mma8452Debug) {
    printf("Setup: mma8452: I2C bus %d successfully opened on address %02x\n",
	   I2C_BUS,MMA8452_ADDRESS);
    printf("Setup: handle = %d\n",handle);
  }
  return MMA8452_SUCCESS;
}
/*! @brief Closes the program in an orderly fashion. Disconnects from 
 * the pigpiod daemon
*/
void mma8452Close(void) {
  pigpio_stop(pi);
}

/*! @brief Checks if the accelerometer's device WHO_AM_I register 
 *         shows the right value of 0x2a
 *  @retval error_code: MMA8452_SUCCESS if the identification was found
 *                         MMA8452_NOT_FOUND otherwise
*/
int mma8452Init(void)
{	
	/*Reading the WHO_AM_I register of the i2c device*/
	
	temp = i2c_read_byte_data(pi,handle, MMA8452_WHO_AM_I);
	if (mma8452Debug)
	  printf("mma8452Init: The value of the WHO_AM_I register is 0x%02x\n", temp);	

	if(temp == MMA8452_ID) {
	  if (mma8452Debug)
		printf("mma8452Init: The accelerometer is online\n");
	}
	else {
	  if (mma8452Debug)	  
		printf("mma8452Init: Could not connect to the device\n");
	  return MMA8452_NOT_FOUND;
	}

	return MMA8452_SUCCESS;

}
      
/*! @brief Function to read raw X,Y,Z acceleration values. The mma8452 is put into active mode
 *  @param pointer to a buffer of 3 int16_t values 
 *         buf[0] = X, buf[1] = Y, buf[2] = z
 *         The acceleration values are signed 12 bit, left adjusted values
 *  @retval an error_code: MMA8452_SUCCESS if successful  
*/
int mma8452ReadXYZ(int16_t*buf)
{
	int8_t  xlow,xhigh,ylow,yhigh,zlow,zhigh;
	int16_t *bufPtr;
	/* 
	   set the device to active mode
	*/
	mma8452SetActive();
	/*
	  check if fast read is selected 
	*/
	bufPtr = buf;
	if (mma8452GetFastRead()) {
	  *bufPtr = ((int16_t)i2c_read_byte_data(pi, handle, MMA8452_OUT_X_MSB))<<8;
	  if (mma8452Debug) 
	    printf("mma8452ReadXYZ: 8 bits: X : Y : Z : 0x%04x",(uint16_t)*bufPtr);
	  bufPtr++;
	  
	  *bufPtr = ((int16_t)i2c_read_byte_data(pi, handle, MMA8452_OUT_Y_MSB))<<8;
	  if (mma8452Debug) 
	    printf(" 0x%04x",(uint16_t)*bufPtr);
	  bufPtr++;

	  *bufPtr = ((int16_t)i2c_read_byte_data(pi, handle, MMA8452_OUT_Z_MSB))<<8;
	  if (mma8452Debug) 
	    printf(" 0x%04x\n",(uint16_t)*bufPtr);
	  return MMA8452_SUCCESS;
	}
	/*
	  normal 12 bit readout
	*/
	else {
	  xhigh = i2c_read_byte_data(pi, handle, MMA8452_OUT_X_MSB);
	  xlow  = i2c_read_byte_data(pi, handle, MMA8452_OUT_X_LSB);

	  *bufPtr = (xhigh << 8) + xlow;
	  if (mma8452Debug) 
	    printf("mma8452ReadXYZ: 12 bits:X : Y : Z : 0x%04x",(uint16_t)*bufPtr);
	  bufPtr++;

	  yhigh = i2c_read_byte_data(pi, handle, MMA8452_OUT_Y_MSB);
	  ylow  = i2c_read_byte_data(pi, handle, MMA8452_OUT_Y_LSB);

	  *bufPtr = (yhigh << 8) + ylow;
	  if (mma8452Debug) 
	    printf(" 0x%04x",(uint16_t)*bufPtr);
	  bufPtr++;

	  zhigh = i2c_read_byte_data(pi, handle, MMA8452_OUT_Z_MSB);
	  zlow  = i2c_read_byte_data(pi, handle, MMA8452_OUT_Z_LSB);

	  *bufPtr = (zhigh << 8) + zlow;
	  if (mma8452Debug) 
	    printf(" 0x%04x\n",(uint16_t)*bufPtr);
	}
	return MMA8452_SUCCESS;
}
/*! @brief Function to read X,Y,Z axes values as integer counts. 
 *        This calls mma8452ReadXYZ and transforms the result to right adjusted signed
 *        integer values
 *  @param buf: pointer to a buffer of 3 int16_t values 
 *         buf[0] = X, buf[1] = Y, buf[2] = z
 *         The acceleration values are signed 12 bit, left adjusted values
 *  @retval error_code: MMA8452_SUCCESS if successful  
*/
int mma8452ReadXYZCounts(int16_t *buf) {
  int16_t tmpBuf[3];
  int16_t *bufPtr;
  int retCode,i;
  /* 
     set the device to active mode
  */
  mma8452SetActive();

  if ((retCode =  mma8452ReadXYZ(tmpBuf)) != MMA8452_SUCCESS)
      return (retCode);
  bufPtr = buf;
  for (i=0;i<3;i++)
    *bufPtr++ = tmpBuf[i] / 16;
  return MMA8452_SUCCESS;
}

/*! @brief Function to read X,Y,Z axes values in units of g. 
 *        This calls mma8452ReadXYZ and calculates the corresponding floating point values
 *        in units if g 
 *  @param fbuf: pointer to a buffer of 3 float values 
 *         buf[0] = X, buf[1] = Y, buf[2] = z
 *         The acceleration values are signed 12 bit, left adjusted values
 *  @retval error_code: MMA8452_SUCCESS if successful  
*/
int mma8452ReadXYZ_G(float *buf) {
  int16_t tmpBuf[3];
  uint16_t bitMask, intMask, signBit=(1<<15);
  int i,vIndex,range,val; 
  int retCode;
  float fval,*bufPtr,mant;
  bool negative;
  /* 
     set the device to active mode
  */
  mma8452SetActive();

  //  printf("Buffer address: 0x%08x\n",(uint32_t) buf);
  if ((retCode =  mma8452ReadXYZ(tmpBuf)) != MMA8452_SUCCESS)
    return (retCode);
  if (mma8452Debug)
    printf("mma8452ReadXYZ_G: X: 0x%04x, Y: 0x%04x, Z: 0x%04x\n",
	   (uint16_t)tmpBuf[0], (uint16_t)tmpBuf[1], (uint16_t)tmpBuf[2]);
  range = mma8452GetRange();
  if (mma8452Debug)
    printf("mma8452ReadXYZ_G; current dynamic range is: %s\n",
	   mma8452Range2String(range));
  /*
    calculate the mask for the integer part 
  */
  intMask = 1;
  for (i=0; i<range; i++)
    intMask = (intMask << 1) | 1;
  intMask <<= 12+2-range;
  /*
    determine the bitmask used to calculate the mantissa
  */
  if (mma8452Debug) {
    bitMask = 1<<(13-range);
    printf("mma8452ReadXYZ_G; integer mask: 0x%04x, bit mask: 0x%04x\n",
	   intMask,bitMask);
  }
  /*
     calculate the acceleration value in units of g
  */
  bufPtr = buf;
  if (mma8452Debug)
    printf("mma8452ReadXYZ_G: acceleration values [g] X,Y,Z: ");

  for (vIndex=0;vIndex < 3; vIndex++) {

    val = tmpBuf[vIndex];
    //    printf("val[%d]: 0x%04x\n",vIndex,(uint16_t)val);
    if (val & signBit) {
      val = -val;
      negative = true;
    }
    else 
      negative = false;

    bitMask = 1<<(13-range);
    mant=0.5;
    fval = (val & intMask) >> (12+2-range);

    for (i=0;i<10-range;i++) {
      if (val & bitMask) {
	fval += mant;
	//	printf("i: %d, mantissa: %f, fval: %f\n",i,mant,fval);
      }
      mant /= 2.0;
      bitMask >>=1;
    }
    /* check the sign bit */
    if (negative)
      fval = -fval;    
    *bufPtr = fval;
    //    printf("mma8452ReadXYZ_G: bufPtr: 0x%08x, value: %f\n",(uint32_t) bufPtr, *bufPtr);
    bufPtr++;
    if (mma8452Debug)
      printf("%f ",fval);

  }
  if (mma8452Debug)
    printf("\n");
  
  return MMA8452_SUCCESS;
}
/*! @brief Checks if the mma8452 is in fast read mode. 
 *  In fast read mode only the 8 most 
 *  significant bits are read out.
 *  @retval fast_mode: true if in fast mode, false otherwise 
*/
bool mma8452GetFastRead() {
  temp = i2c_read_byte_data(pi,handle,MMA8452_CTLR_REG1);
  if (mma8452Debug) {
    if (temp & MMA8452_FREAD)
      printf("mma8452GetFastRead: fast read is enabled\n");
    else
      printf("mma8452GetFastRead: fast read is disabled\n");      
  }
  return (temp & MMA8452_FREAD) >> 1;  
}

/*! @brief Sets or removes mma8452 fast read mode. 
 *         In fast read mode only the 8 most 
 *         significant bits are read out.
 *  @param: fast_mode: true if fast mode is selected, false otherwise
 *  @retval error_code: MMA8452_SUCCESS if successful, an error code otherwise
*/
int mma8452SetFastRead(bool onOff) {
  /*
    to set the fast mode we must be in standby mode
  */
  mma8452SetStandby();

  temp = i2c_read_byte_data(pi,handle,MMA8452_CTLR_REG1);
  if (onOff)
    temp |= MMA8452_FREAD;
  else
    temp &= ~MMA8452_FREAD;
  i2c_write_byte_data(pi,handle,MMA8452_CTLR_REG1,temp);
  if (mma8452Debug)
    mma8452GetFastRead();
  return MMA8452_SUCCESS;
}

/*! @brief Gets the dynamic range The mma8452 has 3 dynamic ranges: 2g, 4g, 8g full scale 
 *  @retval range: a code for the dynamic range: MMA8452_FULL_SCALE_2G, MMA8452_FULL_SCALE_4G, 
 * MMA8452_FULL_SCALE_8G
*/

int mma8452GetRange() {
  temp = i2c_read_byte_data(pi,handle,MMA8452_XYZ_DATA_CFG);
  if (mma8452Debug)
    printf("mma8452GetRange: range is %s\n",
	   mma8452Range2String(temp & MMA8452_FS_MASK));
  return temp & MMA8452_FS_MASK;  
}

/*! @brief Sets the dynamic range The mma8452 has 3 dynamic ranges: 2g, 4g, 8g full scale 
 *  @param range: code for the dynamic range
 *  @retval error_code: MMA8452_SUCCESS if successful, a negative error code
 * otherwise
*/

int mma8452SetRange(int range) {

  mma8452SetStandby();

  if ((range < MMA8452_FULL_SCALE_2G) || (range > MMA8452_FULL_SCALE_8G)) {
    if (mma8452Debug) 
      printf("mma8452SetRange: Invalid range: 0x%02x\n",range);
    return MMA8452_INVALID_PAR;
  }
  temp = i2c_read_byte_data(pi,handle,MMA8452_XYZ_DATA_CFG);
  temp &= ~MMA8452_FS_MASK;
  temp |= range;
  i2c_write_byte_data(pi,handle,MMA8452_XYZ_DATA_CFG,temp);
  
  if (mma8452Debug) {
    temp =  mma8452GetRange();

    switch(range) {
    case MMA8452_FULL_SCALE_2G:
      printf("mma8452SetRange: range is 2g\n");
      break;
    case MMA8452_FULL_SCALE_4G:
      printf("mma8452SetRange: range is 4g\n");
      break;
    case MMA8452_FULL_SCALE_8G:
      printf("mma8452SetRange: range is 8g\n");
      break;
    }
  }
  return MMA8452_SUCCESS;  
}
/*! @brief Converts the dynamic range code into a humanly readable text 
 * @param data rate: the dynamic range code
 * @retval dynamic_range_string: a string describing the dynamic range
 */
char *mma8452Range2String(int range) {
  if ((range < 0) || (range > 2)) {
    if (mma8452Debug)
      printf("mma8452Range2String: invalid range: %d\n",range);
    return NULL;
  }
  return mma8452RangeString[range];
}

/*! @brief Converts the data rate code into a humanly readable text 
 * @param data_rate: the data rate code
 * @retval dynamic_range_string: a string describing the dynamic range
 */
char *mma8452DataRate2String(int dataRate) {
  if ((dataRate < 0) || (dataRate > 7)) {
    if (mma8452Debug)
      printf("mma8452DataRate2String: invalid data rate: %d\n",dataRate);
    return NULL;
  }
  return mm8452DataRangeString[dataRate];
}

/*! @brief Gets the data rate code
 * @retval data_rate: code for the data rate
 */
int mma8452GetDataRate(void) {
  temp = i2c_read_byte_data(pi,handle,MMA8452_CTLR_REG1);
  if (mma8452Debug)
    printf("mma8452GetDataRate: data rate is %s\n",
	   mm8452DataRangeString[(temp >> MMA8452_DR_SHIFT)]);
  return (temp >> MMA8452_DR_SHIFT);
}

/*! @brief Sets the data rate code
 * @param: the data rate code
 * @retval error_code: MMA8452_SUCCESS if successful, a negative error
 * code otherwise
  */
int mma8452SetDataRate(int dataRate) {
  /*
    to set the data rate we must be in standby mode
  */
  if (mma8452GetActiveStandby() != MMA8452_ACTIVE) {
    if (mma8452Debug)
      printf("mma8452SetDataRate: device is in active mode, switch it to standby please\n");
    return MMA8452_NOT_STANDBY;
  } 
  if ((dataRate < 0) || (dataRate > 7)) {
    if (mma8452Debug)
      printf("mma8452DataRate2String: invalid data rate: %d\n",dataRate);
    return MMA8452_INVALID_PAR;
  }
  temp = i2c_read_byte_data(pi,handle,MMA8452_CTLR_REG1);
  temp &= ~(0x7 << MMA8452_DR_SHIFT);
  temp |= dataRate << MMA8452_DR_SHIFT;
  i2c_write_byte_data(pi,handle,MMA8452_CTLR_REG1,temp);
  if (mma8452Debug)
    mma8452GetDataRate();
  return MMA8452_SUCCESS;
}
/*! @brief Converts the error code into a humanly readable text 
 * @param data rate: error code
 * @retval dynamic_range_string: a string describing the error
 */
char *mma8452DataErrorCode2String(int errorCode){
  if ((errorCode > 0) || (errorCode < MMA8452_LAST_ERROR_CODE)){
    if (mma8452Debug) {
      printf("mma8452DataErrorCode2String: Unknown error code\n");
      return NULL;
    }
  }
  return mm8452ErrorString[-errorCode];
}

/*! @brief Gets the running mode
 * @retval running_mode: code for the running mode, which can be standby, wake 
 * or sleep 
 *
  */
int  mma8452GetMode(void){
  temp = i2c_read_byte_data(pi,handle,MMA8452_SYSMOD);
  temp &= MMA8452_SYSMOD_MASK;
  if (mma8452Debug)
    printf("mma8452GetMode: mode is: %s\n",mma8452ModeString[temp]);
  return temp;
}

/*! @brief Converts the running mode code into a humanly readable string
 * @param: code for the running mode
 * @retval running_mode_string: string describing the running mode
  */
char *mma8452Mode2String(int mode) {
  if ((mode < 0) || (mode >2)) {
    if (mma8452Debug)
      printf("mma8452Mode2String: Invalid mode code: %d\n",mode);
    return NULL;
  }
  return mma8452ModeString[mode];
}

/*! @brief Checks if the mma8452 is in standby or active mode
 * @retval active_standby: MMA8452_ACTIVE if active, 
 * MMA8452_STANDBY if in standby mode
 * or sleep 
 *
  */
int mma8452GetActiveStandby() {
  temp = i2c_read_byte_data(pi,handle, MMA8452_CTLR_REG1);
  if (mma8452Debug) {
    if (temp & MMA8452_ACTIVE)
      printf("mma8452GetActiveStandby: active\n");
    else
      printf("mma8452GetActiveStandby: standby\n");
  }
  if (temp & MMA8452_ACTIVE)
    return MMA8542_MODE_ACTIVE;
  else
    return MMA8542_MODE_STANDBY;
}

static int mma8452SetActiveStandby(int mode) {
  temp = i2c_read_byte_data(pi,handle, MMA8452_CTLR_REG1);
  switch (mode) {
  case MMA8542_MODE_ACTIVE:
    temp |=MMA8542_MODE_ACTIVE;
    break;
  case MMA8542_MODE_STANDBY:
    temp &= ~MMA8542_MODE_ACTIVE;
    break;
  default:
    if (mma8452Debug)
      printf(" mma8452SetActiveStandby: Invalid parameter mode: 0x%02x\n",mode);
    return MMA8452_INVALID_PAR;
  }
  i2c_write_byte_data(pi, handle, MMA8452_CTLR_REG1, temp);
  if (mma8452Debug)
    mma8452GetActiveStandby();
  return MMA8452_SUCCESS;
}
/*! @brief Sets the mma8452 to active mode. In this mode acceleration data
 * are aquired
 *  @retval error_code: MMA8452_SUCCESS if successful
 *          an error code otherwise
 */
int mma8452SetActive() {
  return mma8452SetActiveStandby(MMA8542_MODE_ACTIVE);
}

/*! @brief Sets the mma8452 to standby mode. In this mode configuration 
 * registers can be programmed
 *  @retval MMA8452_SUCCESS if successful
 *          an error code otherwise
 */
int mma8452SetStandby() {
  return mma8452SetActiveStandby(MMA8542_MODE_STANDBY);
}

/*! @brief Enables or disables Portrait / Landscape mode. PL mode allows to find
 * out if the device is horizontal or vertical and if is is upside down
 * @param: portrait_landscape: true if PL mode is on, false otherwise
 *  @retval MMA8452_SUCCESS if successful
 *          an error code otherwise
 */
int mma8452EnablePL(bool onOff) {
  /* set the mode to standby allowing to program the PL_CFG register */ 
  mma8452SetStandby();

  temp = i2c_read_byte_data(pi,handle, MMA8452_PL_CFG);
  if (onOff)
    temp |= MMA8452_PL_EN;
  else
    temp &= ~MMA8452_PL_EN;
  i2c_write_byte_data(pi,handle, MMA8452_PL_CFG,temp);
  if (mma8452Debug)
    mma8452IsPLEnabled();
  return MMA8452_SUCCESS;
}

/*! @brief Checks if Portrait / Landscape mode is enabled. PL mode allows to find
 * out if the device is horizontal or vertical and if is is upside down
 * @param: portrait_landscape: true if PL mode is on, false otherwise
 *  @retval portrait_landscape: true if PL mode is on, false otherwise
 *          an error code otherwise
 */

bool mma8452IsPLEnabled() {
  temp = i2c_read_byte_data(pi,handle, MMA8452_PL_CFG);
  temp = (temp & MMA8452_PL_EN) >> 6;
  if (mma8452Debug) {
    if (temp)
      printf("mma8452IsPLEnabled: Portrait / Landscape mode is enabled\n");
    else
      printf("mma8452IsPLEnabled: Portrait / Landscape mode is disabled\n");      
  }
  return (temp);
}

/*! @brief Get a code for the orientation
 *  @retval orientation: code for the orientation, returns -1 of PL mode
 *  is not enabled 
 */
int mma8452GetOrientation(void) {
  if (!mma8452IsPLEnabled())
    return -1;
  temp = i2c_read_byte_data(pi,handle, MMA8452_PL_STATUS);
  temp = (temp & MMA8542_LO_PA_MASK) >> 1;
  if (mma8452Debug)
    printf("mma8452GetOrientation: %s\n",mma8452Orientation2String(temp));
  return temp;	   
}

/*! @brief Converts the orientation code into a humanly readable string
 *  @param: orientation: code for the orientation
 *  
 */
char *mma8452Orientation2String(int orientation) {
  if ((orientation < 0 ) || (orientation > 3)) {
    if (mma8452Debug)
      printf("mma8452Orientation2String: invalid orientation: %d\n",orientation);
    return NULL;
  }
  else
    return mma8452OrientationString[orientation];    
}
