/** \mainpage
*
****************************************************************************
* Copyright (C) Uli Raich
*
* File : mma8452.h
*
* Date : Nov 2018
*
* Revision : 1.0.0
*
* Usage: Sensor Driver for the MMA8452 accelerometer
*
****************************************************************************
*/
/** \file mma8452.h
    \brief Header file for all defined constants and function prototypes
*/
#ifndef _ACC_H_
#define _ACC_H_

#include <stdio.h>
#include <pigpiod_if2.h>
#include <stdint.h>
#include <stdbool.h>

/*!
* @brief I2C Flags are not used but must be given as a parameter to the pigio call
* i2c_open
*/
#define I2C_FLAGS        0
/*!
* @brief I2C bus number
*/
#define I2C_BUS           1

/*!
* @brief I2C device address
*/
#define MMA8452_ADDRESS  0x1c

/*!
* @brief error code for success
*/
#define MMA8452_SUCCESS              0
/*!
* @brief error code when we cannot contact the pigpio daemon
*/
#define MMA8452_PIGPIO_INIT_FAILURE -1
/*!
* @brief error code when we cannot open the I2C bus
*/
#define MMA8452_I2C_INIT_FAILURE    -2
/*!
* @brief error code if no mma8452 chip is connected to the I2C bus
*/
#define MMA8452_NOT_FOUND           -3
/*!
* @brief error code for an invalid parameter
*/
#define MMA8452_INVALID_PAR         -4
/*!
* @brief error code if the mma8452 is not in standby when it should be
*/
#define MMA8452_NOT_STANDBY         -5
/*!
* @brief error code if the mma8452 is not in active mode when it should be
*/
#define MMA8452_NOT_ACTIVE          -6
/*!
* @brief the last error code
*/
#define MMA8452_LAST_ERROR_CODE     -6
/*!
* @brief the mma8452 ID code
*/
#define MMA8452_ID                   0x2a

#define MMA8452_STATUS               0x00
#define MMA8452_OUT_X_MSB            0x01
#define MMA8452_OUT_X_LSB            0x02
#define MMA8452_OUT_Y_MSB            0x03
#define MMA8452_OUT_Y_LSB            0x04
#define MMA8452_OUT_Z_MSB            0x05
#define MMA8452_OUT_Z_LSB            0x06

#define MMA8452_SYSMOD               0x0b
#define MMA8452_INT_SOURCE           0x0c
#define MMA8452_WHO_AM_I             0x0d
#define MMA8452_XYZ_DATA_CFG         0x0e
#define MMA8452_HP_FILTER_CUTOFF     0x0f
#define MMA8452_PL_STATUS            0x10
#define MMA8452_PL_CFG               0x11
#define MMA8452_PL_COUNT             0x12
#define MMA8452_PL_BF_ZCOMP          0x13
#define MMA8452_P_L_THS_REG          0x14
#define MMA8452_FF_MT_CFG            0x15
#define MMA8452_FF_MT_SRC            0x16
#define MMA8452_FF_MT_THS            0x17
#define MMA8452_FF_MT_COUNT          0x18

#define MMA8452_TRANSIENT_CFG        0x1d
#define MMA8452_TRANSIENT_SRC        0x1e
#define MMA8452_TRANSIENT_THS        0x1f
#define MMA8452_TRANSIENT_COUNT      0x20
#define MMA8452_PULSE_CFG            0x21         
#define MMA8452_PULSE_SRC            0x22
#define MMA8452_PULSE_THSX           0x23         
#define MMA8452_PULSE_THSY           0x24
#define MMA8452_PULSE_THSZ           0x25         
#define MMA8452_PULSE_TMLT           0x26
#define MMA8452_PULSE_LTCY           0x27         
#define MMA8452_PULSE_WIND           0x28
#define MMA8452_ASLP_COUNT           0x29
#define MMA8452_CTLR_REG1            0x2a
#define MMA8452_CTLR_REG2            0x2b
#define MMA8452_CTLR_REG3            0x2c
#define MMA8452_CTLR_REG4            0x2d
#define MMA8452_CTLR_REG5            0x2e
#define MMA8452_OFF_X                0x2f
#define MMA8452_OFF_Y                0x30
#define MMA8452_OFF_Z                0x31

/* status register bits */
#define MMA8452_ZYXOW                (0x01 << 7)
#define MMA8452_ZOW                  (0x01 << 6)
#define MMA8452_YOW                  (0x01 << 5)
#define MMA8452_XOW                  (0x01 << 4)
#define MMA8452_ZYXDR                (0x01 << 3)
#define MMA8452_ZDR                  (0x01 << 2)
#define MMA8452_YDR                  (0x01 << 1)
#define MMA8452_XDR                  0x01

/* sysmod bits */
#define MMA8452_STANDBY              0x00
#define MMA8452_WAKE                 0x01
#define MMA8452_SLEEP                0x02

/* interrupt status register bits */
#define MMA8452_SRC_DRDY             0x01
#define MMA8452_SRC_FF_MT            (0x01 << 2)
#define MMA8452_SRC_FF_PULSE         (0x01 << 3)
#define MMA8452_SRC_FF_LNDPRT        (0x01 << 4)
#define MMA8452_SRC_FF_TRANS         (0x01 << 5)
#define MMA8452_SRC_FF_ASLP          (0x01 << 7)

/* XYZ data cfg register bits */

#define MMA8452_FS_MASK               0x03
#define MMA8452_FULL_SCALE_2G         0x00
#define MMA8452_FULL_SCALE_4G         0x01
#define MMA8452_FULL_SCALE_8G         0x02
#define MMA8452_HFP_OUT               (0x01 << 7)

/* high pass filter cutoff bits */
#define MMA8452_SEL0                  0x01
#define MMA8452_SEL1                  (0x01 << 1)
#define MMA8452_PULSE_LPF_EN          (0x01 << 4)
#define MMA8452_PULSE_HPF_BYP         (0x01 << 5)

/* portrait/landscape status register bits */
#define MMA8452_BAFRO                 0x01
#define MMA8452_PORTRAIT_UP           0x00
#define MMA8452_PORTRAIT_DOWN         (0x01 << 1)
#define MMA8452_PORTRAIT_UP           0x00
#define MMA8452_LANDSCAPE_RIGHT       (0x02) << 1)
#define MMA8452_LANDSCAPE_LEFT        (0x03) << 1)

/* portrait/landscape configuration register bits */
#define MMA8452_PL_EN                 (0x01 << 6)
#define MMA8452_DBCNTM                (0x01 << 7)

/* ctrl reg 1 bits */

#define MMA8452_ACTIVE                0x01
#define MMA8452_FREAD                 (0x01 << 1)             
#define MMA8452_LNOISE                (0x01 << 2)
#define MMA8452_DR_SHIFT              3
#define MMA8452_ASLP_RATE0            (0x01 << 6)
#define MMA8452_ASLP_RATE1            (0x01 << 7)

/* sysmod reg bits */
#define MMA8452_SYSMOD_MASK           3 

/* pl status register bits */
#define MMA8542_LO_PA_MASK            0x06

/* run mode */
#define MMA8542_MODE_ACTIVE           1
#define MMA8542_MODE_STANDBY          0

void mma8452SetDebug(bool);
int  mma8452Setup(void);
int  mma8452Init(void);
int  mma8452ReadXYZ(int16_t *);
int  mma8452ReadXYZ_G(float *);
int  mma8452GetActiveStandby(void);
int  mma8452SetStandby();
int  mma8452SetActive();
int  mma8452GetRange(void);
int  mma8452SetRange(int range);
bool mma8452GetFastRead(void);
int  mma8452SetFastRead(bool onOff);
int  mma8452GetDataRate(void);
int  mma8452SetDataRate(int dataRate);
int  mma8452GetMode(void);
int  mma8452EnablePL(bool);
bool mma8452IsPLEnabled(void);
int  mma8452GetOrientation(void);
char *mma8452Orientation2String(int);
char *mma8452DataRate2String(int dataRate);
char *mma8452DataErrorCode2String(int errorCode);
char *mma8452Mode2String(int mode);
char *mma8452Range2String(int range);

#endif //_ACC_H_

