/*! \file mma8452.c 
 *  \brief
 *    Reads out the mma8452 accelerometer using the libmm8542.so shared
 *    library. It 
 *    Copyright U. Raich Nov. 2018
 *    This program is part of a course on embedded systems held
 *    at the University of Cape Coast, Ghana in 2018
 *    It is released under the GNU Public License 
 *    For details please see https://www.gnu.org/licenses/gpl.html
 */
#include <stdio.h>
#include "mma8452.h"

int main(int argc, char **argv) {

  //  int i,j;
  int runMode,range,orientation;
  int16_t xyzBuf[3];
  float xyz_G[3];
  
  /*
    enable debug messages
  */
  mma8452SetDebug(true);

  /*
    connect to the pigpio daemon
  */
  if (mma8452Setup() != MMA8452_SUCCESS) {
    printf("mma8452Test: Could not initialize the mma8452\n");
    return -1;
  }
  /*
    open the I2C bus and check that the mma8452 is connected 
  */
  if (mma8452Init() != MMA8452_SUCCESS) {
    printf("mma8452Test: Could not find the mma8452\n");
    return -1;
   
  }
  /*
    test setting and reading the run mode
    This is just to test the library functions. In fact the calls are not really
    needed as the readout functions automatically set "active" mode while
    the configuration functions set "standby mode"
  */
  runMode = mma8452GetActiveStandby();
  if (runMode == MMA8542_MODE_ACTIVE)
    printf("mma8452Test: Current run mode: active\n");
  else if (runMode == MMA8542_MODE_STANDBY)
    printf("mma8452Test: Current run mode: standby\n");
  else
    printf("mma8452Test: Don't understand the run mode: 0x%02x\n",runMode);
    
  if (mma8452SetActive() != MMA8452_SUCCESS)
    printf("mma8452Test: Cannot set run mode to active\r\n");
  if (mma8452SetStandby() != MMA8452_SUCCESS)
    printf("mma8452Test: Cannot set run mode to standby\r\n");

  /*
    test setting the dynamic range
  */
  range = mma8452GetRange();
  switch(range) {
  case MMA8452_FULL_SCALE_2G:
    printf("mma8452Test: range is 2g\n");
    break;
  case MMA8452_FULL_SCALE_4G:
    printf("mma8452Test: range is 4g\n");
    break;
  case MMA8452_FULL_SCALE_8G:
    printf("mma8452Test: range is 8g\n");
    break;
  }
  if (mma8452SetRange(MMA8452_FULL_SCALE_4G) != MMA8452_SUCCESS)
    printf("mma8452Test: Cannot set range to 4g\r\n");
  /*
    test fast read
  */
  if (mma8452GetFastRead())
    printf("mma8452Test: Fast read is enabled\n");
  else
    printf("mma8452Test: Fast read is disabled\n");
  
  if (mma8452EnablePL(true) != MMA8452_SUCCESS)
    printf("mma8452Test: Could not enable Portrait Landscape mode\n");
  else
    printf("mma8452Test: Portrait Landscape mode is enabled\n");

  if (mma8452SetFastRead(true) != MMA8452_SUCCESS)
    printf("mma8452Test: Cannot set fast read mode\n");
  /* set device to active mode */

  if (mma8452ReadXYZ(xyzBuf) != MMA8452_SUCCESS)
    printf("mma8452Test: Cannot read acceleration data\n");
  else {
    printf("mma8452Test: X: 0x%04x, Y:0x%04x, Z: 0x%04x\n",
	   (uint16_t)xyzBuf[0],(uint16_t)xyzBuf[1],(uint16_t)xyzBuf[2]);
    printf("mma8452Test: X: %d, Y: %d, Z: %d\n",
	   xyzBuf[0]/16,xyzBuf[1]/16,xyzBuf[2]/16);
  }

  if (mma8452ReadXYZ_G(xyz_G) != MMA8452_SUCCESS)
    printf("mma8452Test: Cannot read acceleration data\n");
  else {
    printf("mma8452Test: X: %f, Y: %f, Z: %f\n",
	   xyz_G[0],xyz_G[1],xyz_G[2]);
  }
  /*  
  for (int i=0;i<3;i++)
    printf("main: xvz_[%d]: 0x%08x, value: %f\n",
	   i,(uint32_t)&xyz_G[i],xyz_G[i]);
  */
  /* 
     change the resolution to 8g
  */

  if (mma8452SetRange(MMA8452_FULL_SCALE_8G) != MMA8452_SUCCESS)
    printf("mma8452Test: Cannot set resolution to 8g\r\n");
  else
    printf("mma8452Test: Resolution successfully set to 8g\r\n");

  if (mma8452ReadXYZ_G(xyz_G) != MMA8452_SUCCESS)
    printf("mma8452Test: Cannot read acceleration data\n");
  else {
    printf("mma8452Test: results with 8g resolution: X: %f, Y: %f, Z: %f\n",
	   xyz_G[0],xyz_G[1],xyz_G[2]);
  } 
  orientation = mma8452GetOrientation();
  printf("mma8452Test: orientation: %s\n",
	 mma8452Orientation2String(orientation));
}



